`timescale 1ns / 1ps
/*
 Company: 
 Engineer: James Patchell
 
 Create Date:    17:48:34 01/13/2009 
 Design Name:	Ehternut Glue Logic CPLD 
 Module Name:    ethernutcpld 
 Project Name: Ethernut
 Target Devices: XC9536
 Tool versions: Webpack 9.1
 Description: 

 Dependencies: 

 Revision: 
 Revision 0.01 - File Created
 Additional Comments:
 
Copyright (c) 2009, James Patchell
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
           this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
          this list of conditions and the following disclaimer in the documentation
          and/or other materials provided with the distribution.
    * Neither the name of the <ORGANIZATION> nor the names of its contributors
          may be used to endorse or promote products derived from this software
          without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


*/
module ethernutcpld(ad, adx, a, rd_n, wd_n, ale, sramcs_n, nic_cs_n, sa);
    inout [7:0] ad;
    output [7:0] adx;
    input [7:0] a;
    input rd_n;
    input wd_n;
    input ale;
    output sramcs_n;
    output nic_cs_n;
    output [4:0] sa;

    parameter NICADR    =    16'hc000;
	 parameter BANKREG   =    16'hff00;
	 
	 wire [15:0] adr;
	 reg [4:0] bank;
    reg [7:0] adx;
	 reg nic_cs_n;
	 reg sramcs_n;
	 reg [4:0] sa;
	 reg rdBank;
	 
	 always @ (ale or ad)
		if(ale)
			adx = ad;		//address latch

    assign adr = {a,adx};
	 
    always @ (adr)
		if(adr[15:8] == NICADR[15:8])
			nic_cs_n = 0;	//active low
		else
			nic_cs_n = 1;	//not selected

	always @ (adr or rd_n)
		if((adr[15:8] == BANKREG[15:8]) && (rd_n == 0))
			rdBank = 1;
		else
			rdBank = 0;
			
	assign ad = rdBank?{3'b0,bank}:8'bzzzzzzzz;
	
	//-------------------------
	// ramcs is funny...
	// we deselect ram whenever
	// a15 and 14 are 1
	//-------------------------
    always @ (adr)
		if(adr[15:14] == 2'b11)
			sramcs_n = 1;
		else
			sramcs_n = 0;	//active low ram select
			
    always @ (adr or wd_n or ad)
		if((adr[15:8] == BANKREG[15:8]) && (wd_n == 0))
			bank = ad[4:0];	//latch
			
    always @ (bank or adr)
		if(adr[15])
			sa[0] = !bank[0];
		else
			sa[0] = adr[14];

    always @ (bank or adr)
		if(adr[15])
			sa[1] = !bank[1];
		else
			sa[1] = 0;

    always @ (bank or adr)
		if(adr[15])
			sa[2] = !bank[2];
		else
			sa[2] = 0;

    always @ (bank or adr)
		if(adr[15])
			sa[3] = !bank[3];
		else
			sa[3] = 0;
			
    always @ (bank or adr)
		if(adr[15])
			sa[4] = !bank[4];
		else
			sa[4] = 0;
			
endmodule
